use std::borrow::BorrowMut;

fn directional_scenic_score(seen: &Vec<i16>, height: i16) -> u32 {
    (match seen.iter().rev().position(|x| { *x >= height }) {
        None => seen.len(),
        Some(x) => x + 1,
    }) as u32
}

fn main() {
    const INPUT: &str = include_str!("../input.txt");

    let mut grid: Vec<Vec<(i16, bool, u32)>> = Vec::new();

    for row in INPUT.lines() {
        grid.push(Vec::new());
        let r = grid.last_mut().unwrap();
        for col in row.chars() {
            r.push((col.to_digit(10).unwrap().try_into().unwrap(), false, 1));
        }
    }

    for row in grid.iter_mut() {
        let mut max_seen: i16 = -1;
        let mut seen: Vec<i16> = Vec::new();
        for tree in row.iter_mut() {
            tree.2 *= directional_scenic_score(&seen, tree.0);
            seen.push(tree.0);
            if tree.0 > max_seen {
                max_seen = tree.0;
                tree.1 = true;
            }
        }
        
        max_seen = -1;
        seen.clear();
        for tree in row.iter_mut().rev() {
            tree.2 *= directional_scenic_score(&seen, tree.0);
            seen.push(tree.0);
            if tree.0 > max_seen {
                max_seen = tree.0;
                tree.1 = true;
            }
        }
    }

    for i in 0..grid[0].len() {
        let mut max_seen: i16 = -1;
        let mut seen: Vec<i16> = Vec::new();
        for j in 0..grid.len() {
            let tree = grid[j][i].borrow_mut();
            tree.2 *= directional_scenic_score(&seen, tree.0);
            seen.push(tree.0);
            if tree.0 > max_seen {
                max_seen = tree.0;
                tree.1 = true;
            }
        }

        max_seen = -1;
        seen.clear();
        for j in (0..grid.len()).rev() {
            let tree = grid[j][i].borrow_mut();
            tree.2 *= directional_scenic_score(&seen, tree.0);
            seen.push(tree.0);
            if tree.0 > max_seen {
                max_seen = tree.0;
                tree.1 = true;
            }
        }
    }

    let n_trees_visible = grid.iter().map(|v| -> usize {
        v.iter().filter(|(_, vis, _)| { *vis }).count()
    }).sum::<usize>();
    println!("{n_trees_visible}");

    let max_scenic_score = grid.iter().map(|v| {
        v.iter().map(|(_, _, score)| { score }).max().unwrap()
    }).max().unwrap();
    println!("{max_scenic_score}");
}
